import igraph_testing as ig
import descriptors as ds
import os
import fpdf
import numpy as np
from PIL import Image
import webbrowser
import argparse

current_dir = os.getcwd()
data_path = f"{current_dir}/py_graspi/data/"
descriptors_path = f"{current_dir}/py_graspi/descriptors/"
expected_distances_path = f"{current_dir}/py_graspi/distances/"
results_path = f"{current_dir}/py_graspi/comparisons/"
test_files = [os.path.splitext(file)[0] for file in os.listdir(data_path) if os.path.splitext(file)[0].count("_") > 3]
epsilon = 1e-5


def file_checker(file1, file2, is_idTor):
    idTor_comparison = []
    distance_comparison = []

    with open(file1,'r') as f1, open(file2, 'r') as f2:
        f1_line = f1.readline()
        f2_line = f2.readline()

        f1_line = f1_line.split()
        f2_line = f2_line.split()

        for x in range(len(f1_line)):
            if is_idTor == True:
                if float(f1_line[0]) == float(f2_line[0]):
                    if abs(float(f1_line[-1]) - float(f2_line[-1])) > epsilon or abs(float(f2_line[-2]) - float(f1_line[-2])) > epsilon or abs(float(f2_line[1]) - float(f1_line[1])) > epsilon:
                        idTor_comparison.append(f'{float(f1_line[0])} Expected: {float(f1_line[1])} {float(f1_line[-2])} {float(f1_line[-1])} Actual: {float(f2_line[1])} {float(f2_line[-2])} {float(f2_line[-1])}\n')
        
            else:
                if abs(float(f1_line[0]) - float(f2_line[0])) > epsilon: 
                    distance_comparison.append(f'Line: {x+1} Expected: {float(f1_line[0])} Actual: {float(f2_line[0])}\n')

        
    # filename = file2.split('_',-1)
    filename = file1.rsplit('.',1)[0]
    filename = filename.rsplit('/',1)[1]


    if len(idTor_comparison) != 0 or len(distance_comparison) != 0:
    
        file = open(f"{results_path + filename}_comparison.txt",'w')
        if len(idTor_comparison) != 0:
            file.writelines(idTor_comparison)
        else:
            file.writelines(distance_comparison)
        file.close()


def main():
    for test_file in test_files:
        filename, distanceType = test_file.rsplit('_',1)
        expectedFile = data_path + test_file + ".txt"
        file = f'{expected_distances_path + filename}-{distanceType}.txt' 
        is_idTor = (distanceType[:2] == 'Id')

        file_checker(expectedFile, file, is_idTor)


if __name__ == '__main__':
    main()


# Configuration file for the Sphinx documentation builder.
import os
import sys
sys.path.insert(0, os.path.abspath('C:\\Users\\annaj\\PycharmProjects\\Py-Graspi-Project\\py-graspi\\py_graspi'))

# For the full list of built-in configuration values, see the documentation:
# https://www.sphinx-doc.org/en/master/usage/configuration.html

# -- Project information -----------------------------------------------------
# https://www.sphinx-doc.org/en/master/usage/configuration.html#project-information

project = 'Py_Graspi'
copyright = '2025, Dr. Olga Wodo'
author = 'Dr. Olga Wodo'
release = '0.1.0.0'

# -- General configuration ---------------------------------------------------
# https://www.sphinx-doc.org/en/master/usage/configuration.html#general-configuration

extensions = ['sphinx.ext.autodoc']

templates_path = ['_templates']
exclude_patterns = []



# -- Options for HTML output -------------------------------------------------
# https://www.sphinx-doc.org/en/master/usage/configuration.html#options-for-html-output

html_theme = 'sphinx_rtd_theme'
html_static_path = ['_static']



import sys
import numpy as np
import matplotlib.pyplot as plt
import igraph_testing as ig

def find_coords(filename):
    """
        find coordinates of file

        Args:
            filename(string): path to file

        Returns:
            coords: x,y,x coords of file in a list.
        """
    with open(filename, "r") as file:
        header = file.readline().split(' ')
        dimX, dimY = int(header[0]), int(header[1])
        if len(header) < 3:
            dimZ = 1
        else:
            if int(header[2]) == 0:
                dimZ = 1
            else:
                dimZ = int(header[2])

        if dimZ > 1:
            # dimZ = dimX * dimY
            is_2d = False
        coords = [dimX, dimY, dimZ]
    return coords

def filterGraph(graph):
    keptEdges = [edge for edge in graph.get_edgelist()
                 if graph.vs[edge[0]]['color'] == graph.vs[edge[1]]['color']
                 or 'red' in {graph.vs[edge[0]]['color'], graph.vs[edge[1]]['color']}
                 or 'blue' in {graph.vs[edge[0]]['color'], graph.vs[edge[1]]['color']}]

    return graph.subgraph_edges(keptEdges, delete_vertices=False)

def find_BTR_tortuosity(g, is_2d, filename):
    """
        finds and visualizes the tortuosity of Black to Red vertices
        Args:
            g (ig.Graph): The input graph.
            is_2d(bool): true if 2D of false if otherwise
            filename(string): path to file

        Returns:
            No return but a visualization is created and displayed on screen.
        """
    numVertices = g.vcount()
    redVertex = g.vcount() - 2
    blackToRedList = []
    filteredGraph = filterGraph(g)
    idOfPixelIn1DArray, tort = read_BTR_file_and_extract_numbers(filename)
    #Calculate vertex frequencies
    vertex_frequency = [0] * numVertices
    for i in range(len(idOfPixelIn1DArray)):
        vertex_frequency[idOfPixelIn1DArray[i]] = tort[i]

    vertex_frequency = vertex_frequency[:-3]
    dimX,dimY,dimZ = coords = find_coords(filename)
    data_2d = np.array(vertex_frequency).reshape(dimY, dimX)

    # Create the heatmap
    #plt.rcParams.update({'font.size': 20})
    plt.title("Black to Red Tortuosity HeatMap")
    plt.imshow(data_2d, cmap='hsv', interpolation='nearest')
    plt.colorbar()  # Add a colorbar to show the values
    plt.show()

def find_WTB_tortuosity(g, is_2d, filename):
    """
        finds and visualizes the tortuosity of White to Black vertices
        Args:
            g (ig.Graph): The input graph.
            is_2d(bool): true if 2D of false if otherwise
            filename(string): path to file

        Returns:
            No return but a visualization is created and displayed on screen.
        """
    numVertices = g.vcount()
    redVertex = g.vcount() - 2
    blueVertex = g.vcount() - 1
    whiteToBlueList = []
    idOfPixelIn1DArray, tort = read_WTB_file_and_extract_numbers(filename)
    #Calculate vertex frequencies
    vertex_frequency = [0] * numVertices
    for i in range(len(idOfPixelIn1DArray)):
        vertex_frequency[idOfPixelIn1DArray[i]] = tort[i]

    vertex_frequency = vertex_frequency[:-3]
    dimX,dimY,dimZ = coords = find_coords(filename)
    data_2d = np.array(vertex_frequency).reshape(dimY, dimX)

    # Create the heatmap
    #plt.rcParams.update({'font.size': 20})
    plt.title("White to Blue Tortuosity HeatMap")
    plt.imshow(data_2d, cmap='hsv', interpolation='nearest')
    plt.colorbar()  # Add a colorbar to show the values
    plt.show()

# Define the function to read the file and extract the numbers
def read_BTR_file_and_extract_numbers(base_filename):
    """
        reads Black to Red Tortuosity ID file and extracts numbers needed from it
        Args:
            base_filename(string): path to file

        Returns:
            idOfPixelIn1DArray(list): list of id of pixel in 1D array
            tort(list): list of tortuosity, index matches that of idOfPixelIn1DArray
        """
    base_filename = base_filename[5:-4]
    file_path = f"distances/{base_filename}-IdTortuosityBlackToRed.txt"
    idOfPixelIn1DArray = []
    tort = []
    # Open the file in read mode
    with open(file_path, "r") as file:
        # Read each line in the file
        for line in file:
            # Split the line into a list of strings
            parts = line.split()
            # Extract the first and second numbers and convert them to appropriate types
            first_number = int(parts[0])
            second_number = float(parts[1])
            # Append the numbers to their respective lists
            idOfPixelIn1DArray.append(first_number)
            tort.append(second_number)

    return idOfPixelIn1DArray, tort

def read_WTB_file_and_extract_numbers(base_filename):
    """
        reads White to Black Tortuosity ID file and extracts numbers needed from it
        Args:
            base_filename(string): path to file

        Returns:
            idOfPixelIn1DArray(list): list of id of pixel in 1D array
            tort(list): list of tortuosity, index matches that of idOfPixelIn1DArray
        """
    base_filename = base_filename[5:-4]
    file_path = f"distances/{base_filename}-IdTortuosityWhiteToBlue.txt"
    idOfPixelIn1DArray = []
    tort = []

    # Open the file in read mode
    with open(file_path, "r") as file:
        # Read each line in the file
        for line in file:
            # Split the line into a list of strings
            parts = line.split()
            # Extract the first and second numbers and convert them to appropriate types
            first_number = int(parts[0])
            second_number = float(parts[1])
            # Append the numbers to their respective lists
            idOfPixelIn1DArray.append(first_number)
            tort.append(second_number)

    return idOfPixelIn1DArray, tort

def main():
    filename = sys.argv[1]
    (g, is_2D, black_vertices, white_vertices, black_green, black_interface_red, white_interface_blue, dim, interface_edge_comp_paths,
     shortest_path_to_red, shortest_path_to_blue, CT_n_D_adj_An, CT_n_A_adj_Ca)  = ig.generateGraphAdj(filename)
    find_BTR_tortuosity(g, is_2D, filename)
    find_WTB_tortuosity(g, is_2D, filename)

if __name__ == '__main__':
    main()